#!/usr/bin/python
# -*- coding: utf-8 -*-
#"""
#KOPR 1.0
#"""

import sys
from math import *
import numpy as np
import comephem
import alb
import koprclass
import koprhand
import datetime
import datavar
import time

# ********** Basic functions ***********
# these functions are to make code easier to read
def updItems(mainlst, newlst, colors):
    for line in mainlst: # delete returns
      if line[0] not in newlst:
        mainlst.remove(line)
    colors = [x for x in colors if x not in [item[2] for item in mainlst]] # delete used colours
    for line in newlst: # add new returns
      if line not in [item[0] for item in mainlst]:
        mainlst.append( [line, 1, colors[-1]] )
        colors.pop()
    return mainlst

def updObs(mainlst, newlst):
    for line in mainlst: # delete returns
      if line[0] not in newlst:
        mainlst.remove(line)
    for line in newlst: # add new return
      if line not in [item[0] for item in mainlst]:
        mainlst.append( [line, 1] )
    return mainlst

def ObsScatter( dt, mag, dist, logr, mp_arrray, rng, distcorr ):
    h0, n = 4., 4.

    if distcorr == 1:
      distcorr = 0
    else: 
      distcorr = 1
    
    for item in mp_arrray: # find magnitude parameters
      if item[2] < dt < item[3]:
        h0, n = item[0], item[1] 
    calcmag = h0 + 5 * log10(dist)*distcorr + 2.5 * n * logr # calculate model mag
    if (calcmag - rng) < mag < (calcmag + rng): # check if magnitude is in range
      return True  
    else:
      return False  

def perilen(e, q):
    perilen = 19133.
    if e < 1:
      a = q / ( 1. - e )
      if a < 14:
        perilen = sqrt(a*a*a)*365.25
    return perilen/2.      

def DTNorm(dt, halfw):
    if fabs(dt) > halfw:
      dt = copysign(halfw,dt)
    return dt

def FindPeriele(peridict, year):
    for key in peridict:
      line = peridict[key]
      if line[0] == year[0]:
        return line[1], key
        break

def GetRange(ranges, tjday, q, e):
    if ranges[0] == 0:
      range1 = ranges[1]-tjday
      range2 = ranges[2]-tjday 
    elif ranges[0] == 1:
      range1 = ranges[1]
      range2 = ranges[2] 
    elif ranges[0] == 2:
      range1 = - alb.finddt(ranges[1],q,e)
      range2 = alb.finddt(ranges[2],q,e)
    elif ranges[0] == 3:
      range1 = - alb.finddt(pow(ranges[1],10),q,e)
      range2 = alb.finddt(pow(ranges[2],10),q,e)
    return range1, range2
      
# ********* Main functions **************
# This are main functions to be executed in other files
def sort(obs_lines, kometa, new, observers = [], returns = []):
    obs_list, ret_list, array, peridict, pjdays = [], [], [], {}, []

    if not returns:
      for item in kometa.perihelions:
        returns.append( [item.split('/')[0], 0, ''] )

    for item in kometa.perihelions:
      tjday = comephem.eDatetoJday(item)
      pjdays.append(tjday)
      peridict[tjday] = [ item.split('/')[0], kometa.elements[item] ]

    pjdays = np.array(pjdays)
    
    for line in obs_lines:
     if line:
      obs = koprclass.icqcode()
      obs.decode_icq(line)
      try:
        jday = comephem.eDatetoJday(obs.date)
        tjday = pjdays[(np.abs(pjdays-jday)).argmin()] # find nearest perihelion date
      except:
        pass #print line
      
      if peridict[tjday][0] not in ret_list:
        ret_list.append(peridict[tjday][0])

      if obs.otype not in obs_list:
        obs_list.append(obs.obscode)

      calc = comephem.ephem()
      calc.readdb(peridict[tjday][1])
      try:
        calc.compute(obs.date, '00:00', ['comet', 'extend'])
      except:
        print line
      #               0-jdate    1-dt    2-r       3-logr        4-return          5-type    6-obscode     7-mag      8-dist   9-phase       10-unc       11-neg
      array.append( [ jday, jday-tjday, calc.r, log10(calc.r), peridict[tjday][0], obs.otype, obs.obscode, obs.mag, calc.dist, calc.phase, obs.uncsign, obs.negsign ] )      
    
    if new == 1:
      returns = [x for x in returns if x[0] not in ret_list]
      updItems(returns, ret_list, datavar.Colorlist())
    
    return array, peridict, updObs(observers, obs_list), returns
      
def getranges(array, index):
    magr = [item[7] for item in array]
    if index == 0:
      lst = [item[index] for item in array]
      return comephem.JdaytoeDate(min(lst)), comephem.JdaytoeDate(max(lst)), str( round(max(magr),1) ), str( round(min(magr),1) ) 
    else:
      lst = [item[index] for item in array]
      return str( round(min(lst),3) ), str( round(max(lst),3) ), str( round(max(magr),1) ), str( round(min(magr),1) )     

def StrNumRanges(ranges, limits):
    xmin, xmax, ymin, ymax = limits
    if ranges[0] == 0:
      xmin, xmax, ymin, ymax = koprhand.timecvt(str(xmin)), koprhand.timecvt(str(xmax)), float(ymin), float(ymax)
    else:
      xmin, xmax, ymin, ymax = float(xmin), float(xmax), float(ymin), float(ymax)
    return xmin, xmax, ymin, ymax  

def NumRanges(ranges, array, compare):
    magr = [item[7] for item in array[0]]
    if ranges[0] == 0:
      lst = [item[ranges[0]] for item in array[0]]
      xmin, xmax, ymin, ymax = comephem.JdaytoDtime(min(lst)), comephem.JdaytoDtime(max(lst)), round(max(magr),1), round(min(magr),1)     
    else:
      lst = [item[ranges[0]] for item in array[0]]
      xmin, xmax, ymin, ymax = round(min(lst),3), round(max(lst),3), round(max(magr),1), round(min(magr),1)    

    if compare:
      try:  
        magr = [item[7] for item in array[1]]
        if ranges[0] == 0:
          lst = [item[ranges[0]] for item in array[1]]
          xmin0, xmax0, ymin0, ymax0 = comephem.JdaytoDtime(min(lst)), comephem.JdaytoDtime(max(lst)), round(max(magr),1), round(min(magr),1)     
        else:
          lst = [item[ranges[0]] for item in array[1]]
          xmin0, xmax0, ymin0, ymax0 = round(min(lst),3), round(max(lst),3), round(max(magr),1), round(min(magr),1)    

        if xmin0 < xmin:
          xmin = xmin0
        if xmax0 > xmax:
          xmax = xmax0
        if ymin0 > ymin:
          ymin = ymin0
        if ymax0 < ymax:
          ymax = ymax0
      except ValueError:
        pass
        
    return  xmin, xmax, ymin, ymax     

def magparams(magparams):
    mp_arrray = []
    for item in magparams:
      item = item.split(',')
      mp_arrray.append( [ float(item[1]), float(item[2]), float(item[3].replace('inf','-9999')), float(item[4].replace('inf','9999')), '-'] )
    return mp_arrray

def filtered_obs(mp_arrray, array, peridict, settings, types, obscode, returns, ranges, corr):
    if 1 in corr[:-1]: # magnitude corrections
      for line in array:   #     distance correction            phase correction                forward scaterring correction
        line[7] = line[7] - 5*log10(line[8]) * corr[0] + alb.phasemag(line[9]) * corr[1] - alb.fwscmag(line[9],float(settings[8]),0.9,-0.6,0.95) * corr[2]
#        line[7] = line[7] - 5*log10(line[8]) * corr[0] + alb.phasemag(line[9]) * corr[1] - alb.fwscmag(line[9],1.0,0.9,-0.6,0.95) * corr[2]
    
    if corr[-1] == 1: # personal coefficients
      personalcoef = koprhand.LoadPersonalCoef()
      for line in array:
        if line[6] in personalcoef.keys() and line[5] == 'vis':
          line[7] = line[7] - personalcoef[line[6]]
    
    # 1. definitive filtering
    returns, obscode, types = [x[0] for x in returns if x[1] == 1], [x[0] for x in obscode if x[1] == 1], [x[0] for x in types if x[1] == 1]
    filtered_array = [ [], [] ]
    #1.1. filter by ranges
    if ranges[0] == 0: # date range
      ranges[1], ranges[2] = comephem.eDatetoJday(str(ranges[1]))-1.0, comephem.eDatetoJday(str(ranges[2]))+1.0
    elif ranges[0] == 1: # dt
      ranges[1], ranges[2] = float(ranges[1])-0.1, float(ranges[2])+0.1
    elif ranges[0] == 2: # r
      ranges[1], ranges[2] = float(ranges[1])-0.01, float(ranges[2])+0.01
    elif ranges[0] == 3: # logr
      ranges[1], ranges[2] = float(ranges[1])-0.001, float(ranges[2])+0.001
      
    array = [obs for obs in array if ranges[2] >= obs[ranges[0]] >= ranges[1] ] 
    forbidden = [] # exclude negative and uncertain obs
    if settings[0] == 1:
      forbidden.append(':')
    if settings[1] == 1:
      forbidden.append('[')
    
    #1.2. filter by perihelions and forbidden signs
    if forbidden:
      array = [obs for obs in array if (obs[4] in returns and obs[10] not in forbidden and obs[11] not in forbidden) ]
    else:
      array = [obs for obs in array if obs[4] in returns ]

    #2. comparable filtering / filter by types / filter by observers
    if settings[2] == 1:   
      filtered_array[0] = [obs for obs in array if (obs[5] in types and obs[6] in obscode and ObsScatter( obs[1], obs[7], obs[8], obs[3], mp_arrray, settings[3], corr[0] )) ]
    else:                                                                                             
      filtered_array[0] = [obs for obs in array if (obs[5] in types and obs[6] in obscode) ]
    filtered_array[1] = [obs for obs in array if obs not in filtered_array[0] ]

    return filtered_array
    
    # result is list of years or list of pre and post perihelion 
def PlotLists( filtered_array, types, returns, perasy, compare, ranges):
    plot_points, obs_xplot, obs_yplot = [], [], []
    
    if compare == 0 and perasy == 0: # case 1 - for every year, every shape
        for year in returns:
          if year[1] == 1:
            for cat in types: 
              if ranges[0] == 0:
                obs_xplot = [ comephem.JdaytoDtime(obs[ranges[0]]) for obs in filtered_array[0] if (obs[4] == year[0] and obs[5] == cat[0])  ]  
                obs_yplot = [ obs[7] for obs in filtered_array[0] if (obs[4] == year[0] and obs[5] == cat[0])  ]
              else:
                obs_xplot = [ obs[ranges[0]] for obs in filtered_array[0] if (obs[4] == year[0] and obs[5] == cat[0])  ] 
                obs_yplot = [ obs[7] for obs in filtered_array[0] if (obs[4] == year[0] and obs[5] == cat[0])  ]
              plot_points.append( [ obs_xplot, obs_yplot, cat[2].split(',')[0], year[2]] )  
    
    elif compare == 1 and perasy == 0: # case 2 - only compare   
        for cat in types: 
          if ranges[0] == 0:
            obs_xplot = [ comephem.JdaytoDtime(obs[ranges[0]]) for obs in filtered_array[0] if obs[5] == cat[0] ]  
            obs_yplot = [ obs[7] for obs in filtered_array[0] if obs[5] == cat[0] ]
          else:
            obs_xplot = [ obs[ranges[0]] for obs in filtered_array[0] if obs[5] == cat[0] ] 
            obs_yplot = [ obs[7] for obs in filtered_array[0] if obs[5] == cat[0] ]
          plot_points.append( [ obs_xplot, obs_yplot, cat[2].split(',')[0], 'black'] )  

        for cat in types: 
          if ranges[0] == 0:
            obs_xplot = [ comephem.JdaytoDtime(obs[ranges[0]]) for obs in filtered_array[1] if obs[5] == cat[0] ]  
            obs_yplot = [ obs[7] for obs in filtered_array[1] if obs[5] == cat[0] ]
          else:
            obs_xplot = [ obs[ranges[0]] for obs in filtered_array[1] if obs[5] == cat[0] ] 
            obs_yplot = [ obs[7] for obs in filtered_array[1] if obs[5] == cat[0] ]
          plot_points.append( [ obs_xplot, obs_yplot, cat[2].split(',')[0], 'red'] )  

    elif compare == 0 and perasy == 1: # case 3 - only perasy   
        for cat in types: 
          if ranges[0] == 0:
            obs_xplot = [ comephem.JdaytoDtime(obs[ranges[0]]) for obs in filtered_array[0] if (obs[1] <= 0 and obs[5] == cat[0]) ]  
            obs_yplot = [ obs[7] for obs in filtered_array[0] if (obs[1] <= 0 and obs[5] == cat[0]) ]
          else:
            obs_xplot = [ obs[ranges[0]] for obs in filtered_array[0] if (obs[1] <= 0 and obs[5] == cat[0]) ] 
            obs_yplot = [ obs[7] for obs in filtered_array[0] if (obs[1] <= 0 and obs[5] == cat[0]) ]
          plot_points.append( [ obs_xplot, obs_yplot, cat[2].split(',')[0], 'black'] )  

        for cat in types: 
          if ranges[0] == 0:
            obs_xplot = [ comephem.JdaytoDtime(obs[ranges[0]]) for obs in filtered_array[0] if (obs[1] > 0 and obs[5] == cat[0]) ]  
            obs_yplot = [ obs[7] for obs in filtered_array[0] if (obs[1] > 0 and obs[5] == cat[0]) ]
          else:
            obs_xplot = [ obs[ranges[0]] for obs in filtered_array[0] if (obs[1] > 0 and obs[5] == cat[0]) ] 
            obs_yplot = [ obs[7] for obs in filtered_array[0] if (obs[1] > 0 and obs[5] == cat[0]) ]
          plot_points.append( [ obs_xplot, obs_yplot, cat[2].split(',')[0], 'red'] )  

    elif compare == 1 and perasy == 1: # case 4 - perasy and compare   
        for cat in types: 
          if ranges[0] == 0:
            obs_xplot = [ comephem.JdaytoDtime(obs[ranges[0]]) for obs in filtered_array[0] if (obs[1] <= 0 and obs[5] == cat[0]) ]  
            obs_yplot = [ obs[7] for obs in filtered_array[0] if (obs[1] <= 0 and obs[5] == cat[0]) ]
          else:
            obs_xplot = [ obs[ranges[0]] for obs in filtered_array[0] if (obs[1] <= 0 and obs[5] == cat[0]) ] 
            obs_yplot = [ obs[7] for obs in filtered_array[0] if (obs[1] <= 0 and obs[5] == cat[0]) ]
          plot_points.append( [ obs_xplot, obs_yplot, cat[2].split(',')[0], 'black'] )  

        for cat in types: 
          if ranges[0] == 0:
            obs_xplot = [ comephem.JdaytoDtime(obs[ranges[0]]) for obs in filtered_array[1] if (obs[1] <= 0 and obs[5] == cat[0]) ]  
            obs_yplot = [ obs[7] for obs in filtered_array[1] if (obs[1] <= 0 and obs[5] == cat[0]) ]
          else:
            obs_xplot = [ obs[ranges[0]] for obs in filtered_array[1] if (obs[1] <= 0 and obs[5] == cat[0]) ] 
            obs_yplot = [ obs[7] for obs in filtered_array[1] if (obs[1] <= 0 and obs[5] == cat[0]) ]
          plot_points.append( [ obs_xplot, obs_yplot, cat[2].split(',')[0], 'gray'] )  

        for cat in types: 
          if ranges[0] == 0:
            obs_xplot = [ comephem.JdaytoDtime(obs[ranges[0]]) for obs in filtered_array[0] if (obs[1] > 0 and obs[5] == cat[0]) ]  
            obs_yplot = [ obs[7] for obs in filtered_array[0] if (obs[1] > 0 and obs[5] == cat[0]) ]
          else:
            obs_xplot = [ obs[ranges[0]] for obs in filtered_array[0] if (obs[1] > 0 and obs[5] == cat[0]) ] 
            obs_yplot = [ obs[7] for obs in filtered_array[0] if (obs[1] > 0 and obs[5] == cat[0]) ]
          plot_points.append( [ obs_xplot, obs_yplot, cat[2].split(',')[0], 'red'] )  
        
        for cat in types: 
          if ranges[0] == 0:
            obs_xplot = [ comephem.JdaytoDtime(obs[ranges[0]]) for obs in filtered_array[1] if (obs[1] > 0 and obs[5] == cat[0]) ]  
            obs_yplot = [ obs[7] for obs in filtered_array[1] if (obs[1] > 0 and obs[5] == cat[0]) ]
          else:
            obs_xplot = [ obs[ranges[0]] for obs in filtered_array[1] if (obs[1] > 0 and obs[5] == cat[0]) ] 
            obs_yplot = [ obs[7] for obs in filtered_array[1] if (obs[1] > 0 and obs[5] == cat[0]) ]
          plot_points.append( [ obs_xplot, obs_yplot, cat[2].split(',')[0], 'lightsalmon'] )  
        
    return plot_points

    # lines for every year for each magnitude parameter    
def PlotLines(mp_arrray, peridict, returns, perasy, ranges, corr): 
    if corr == 1:
      corr = 0
    else:
      corr = 1.
    
    plot_lines, obs_xplot, obs_yplot = [], [], []
    
    if perasy == 1:
      mp_arrray0 = []
      for line in mp_arrray:
        if line[2] < 0 and line[3] > 0:
          mp_arrray0.append( [ line[0], line[1], line[2], 0      , '-' ] )
          mp_arrray0.append( [ line[0], line[1], 0      , line[3], '-' ] )   
        else:
          mp_arrray0.append(line)
      mp_arrray = mp_arrray0     
    
    for year in returns:
      if year[1] == 1:
          for param in mp_arrray:
            eline, tjday = FindPeriele(peridict, year)
            calc = comephem.ephem()
            calc.readdb(eline)
            halfw = perilen(calc.e, calc.q)
            
            if param[2] < 0 and perasy == 1:
              color = 'black'
            if param[2] >= 0 and perasy == 1:  
              color = 'red'
            
            dt_beg, dt_end = param[2], param[3]
            if dt_beg < -halfw: # normalisation of infinite elements to actual range
              dt_beg = -halfw
            if dt_end > halfw: # normalisation of infinite elements to actual range
              dt_end = halfw

            range1, range2 = GetRange(ranges, tjday, calc.q, calc.e)

            if dt_beg < range1:
              dt_beg = range1
            if dt_end > range2:
              dt_end = range2

            step = dt_end - dt_beg  # number of steps in days
            if step > 0:
              start = koprhand.timecvt(comephem.JdaytoeDate(tjday + dt_beg))  # computation start date
              now, step, obs_xplot, obs_yplot = start, int(step)+1, [], []
              
              for i in range(step):
                calc.compute(koprhand.cvt2time(now), '00:00', ['comet', 'extend'])
                dt = alb.jday( now.year, now.month, now.day, float(now.hour)+now.minute/24. ) - tjday
                result = [ now, dt, calc.r, log10(calc.r) ]
                
                obs_xplot.append( result[ranges[0]] )
                obs_yplot.append( param[0] + 5*log10(calc.dist)*corr + 2.5*param[1]*log10(calc.r) )
                now += datetime.timedelta(days = 1)
    
              if perasy == 1:
                plot_lines.append( [ obs_xplot, obs_yplot, color, param[4]] )
              else:
                plot_lines.append( [ obs_xplot, obs_yplot, year[2], param[4]] )
        
    return plot_lines                        
    
def RangeConversion(kometa, returns, peridict, working_list, ranges , compare):
    pjdays, calc, enabled, rdists = np.array(peridict.keys()), comephem.ephem(), [x for x in returns if x[1] == 1], []
    matrix = [ [ '', 0, 0, 0 ],  
               [ '', 0, 0, 0 ] ] #result matrix:    
    
    elements, tjday = FindPeriele(peridict, enabled[0])
    tjday = float(tjday)
    calc.readdb(elements)
    halfw = perilen(calc.e, calc.q)
    
    if ranges[2] == 0: # given date
      beg = comephem.eDatetoJday(ranges[0])
      calc.compute(ranges[0], '00:00', ['comet'])
      dt1 = DTNorm(beg - tjday, halfw)
      rdists.append(calc.r)   
    
      end = comephem.eDatetoJday(ranges[1])
      calc.compute(ranges[1], '00:00', ['comet'])
      dt2 = DTNorm(end - tjday, halfw)
      rdists.append(calc.r)

      if dt1 < 0 and dt2 > 0: # comet is passing perihelion during interval
        rdists.append(calc.q) 
      
      matrix[0] = [ ranges[0], dt1, min(rdists), log10(min(rdists)) ]
      matrix[1] = [ ranges[1], dt2, max(rdists), log10(max(rdists)) ]
      
    elif ranges[2] == 1: # given dt
      dt1 = DTNorm(float(ranges[0]), halfw)
      beg = comephem.JdaytoeDate(dt1 + tjday)
      calc.compute(beg, '00:00', ['comet'])
      rdists.append(calc.r)
      
      dt2 = DTNorm(float(ranges[1]), halfw)
      end = comephem.JdaytoeDate(dt2 + tjday)
      calc.compute(end, '00:00', ['comet'])
      rdists.append(calc.r)
      
      if float(ranges[0]) < 0 and float(ranges[1]) > 0: # comet is passing perihelion during interval
        rdists.append(calc.q)         

      matrix[0] = [ beg, ranges[0], min(rdists), log10(min(rdists)) ]
      matrix[1] = [ end, ranges[1], max(rdists), log10(max(rdists)) ]
      
    elif ranges[2] == 2: # given r 
      dt = alb.finddt( float(ranges[1]), calc.q, calc.e )
      beg = comephem.JdaytoeDate( tjday - dt )
      end = comephem.JdaytoeDate( tjday + dt )
      
      matrix[0] = [ beg, - dt, ranges[0], log10(float(ranges[0])) ]
      matrix[1] = [ end, + dt, ranges[1], log10(float(ranges[1])) ]
      
    elif ranges[2] == 3: # given logr
      dt = alb.finddt( pow(10,float(ranges[1])), calc.q, calc.e )
      beg = comephem.JdaytoeDate( tjday - dt )
      end = comephem.JdaytoeDate( tjday + dt )
      
      matrix[0] = [ beg, - dt, pow(10,float(ranges[0])), ranges[0] ]
      matrix[1] = [ end, + dt, pow(10,float(ranges[1])), ranges[1] ]

    if compare:
      obsrange1, obsrange2 = getranges(working_list[0]+working_list[1], ranges[3])[:2]
    else:
      obsrange1, obsrange2 = getranges(working_list[0], ranges[3])[:2]
    
    if len(enabled)>1:
      if ranges[3] == 0:
          matrix[0][ranges[3]] = obsrange1
          matrix[1][ranges[3]] = obsrange2
      else:
          matrix[0][ranges[3]] = float(obsrange1)
          matrix[1][ranges[3]] = float(obsrange2)
      
    return matrix    
    
def Analysis(working_list, settings):
    array = [ [], [] ]
    output = []
    
    if settings[4] == 0 and settings[9] == 0: # normal
        array[0] = [ (x[3], x[7] ) for x in working_list] 
        xlist,ylist = zip(*array[0])
        result, std, _, _, _  = np.polyfit(xlist,ylist,1, full=True)   
        output.append( [result[1], result[0]/2.5, float(std)/len(ylist), len(ylist), min([x[1] for x in working_list]), max([x[1] for x in working_list])] )

    elif settings[4] == 0 and settings[9] == 1: # fixed slope
        array[0] = [ x[7] - 2.5*settings[10]*x[3] for x in working_list] 
        output.append( [np.mean(array[0]), settings[10], np.std(array[0]), len(array[0]), min([x[1] for x in working_list]), max([x[1] for x in working_list])] )
    
    else: # perasy    
      try:
        array[0] = [ (x[3], x[7] ) for x in working_list if x[1] <= 0 ] 
        array[1] = [ (x[3], x[7] ) for x in working_list if x[1] > 0 ] 

        if array[0]:
          xlist,ylist = zip(*array[0])
          result, std, _, _, _  = np.polyfit(xlist,ylist,1, full=True)   
          output.append( [result[1], result[0]/2.5, float(std)/len(ylist), len(ylist), min([x[1] for x in working_list]), 0] )

        if array[1]:
          xlist,ylist = zip(*array[1])
          result, std, _, _, _  = np.polyfit(xlist,ylist,1, full=True)   
          output.append( [result[1], result[0]/2.5, float(std)/len(ylist), len(ylist), 0, max([x[1] for x in working_list])] )
      except TypeError:
        print 'TypeError in working_list'
        for item in working_list:
          print item
    
    return output         

def GetPlineX(eline, option):
    calc = comephem.ephem()
    calc.readdb(eline[1])
    if option == 0:
      xcoord = comephem.DatetoDatetime(calc.tyear,calc.tmonth,calc.tday)
    elif option == 1:
      xcoord = 0
    elif option == 2:
      xcoord = calc.q
    elif option == 3:
      xcoord = log10(calc.q)

    return xcoord

def GetListLim(work_list, i, compare):
    visible = [x[i] for x in work_list[0]]
    hidden = [x[i] for x in work_list[0]]
#    margin = [ datetime.timedelta(days = 10), 10, 0.1, 0.01 ]

    if compare == 1:
      visible += hidden
    if i == 0:
      return (comephem.JdaytoDtime(min(visible)), comephem.JdaytoDtime(max(visible)))
    else:
      return (min(visible), max(visible))

def GetDefaultLim(option, returns, peridict):
    returns = {x[0]: x[2] for x in returns if x[1] == 1}
    elines = []
    for key in peridict:
      if peridict[key][0] in returns.keys():
        elines.append(peridict[key][1])

    qs, dates = [], []
    for eline in elines:
      calc = comephem.ephem()
      calc.readdb(eline)
      dates.append(comephem.DatetoDatetime(calc.tyear,calc.tmonth,calc.tday))
      qs.append(calc.q)

    if option == 0:
      return min(dates) - datetime.timedelta(days = 100), max(dates) + datetime.timedelta(days = 100)
    elif option == 1:
      return -100, 100
    elif option == 2:
      return min(qs). max(qs)+0.5
    elif option == 3:
      return log10(min(qs)). log10(max(qs)+0.5)

