#!/usr/bin/python
# -*- coding: utf-8 -*-
#"""
#KOPR 1.0
#"""

import sys
import koprhand
import koprdial
import koprfunc
import offline
import koprfig
import photom
import koprclass
import comephem
import numpy as np
import datavar
from PyQt4 import QtCore, QtGui
#import matplotlib
#matplotlib.use('Qt4Agg')
from matplotlib import pyplot as plt
from matplotlib.widgets import Slider, Button, RadioButtons
from matplotlib.backends.backend_qt4agg import NavigationToolbar2QT as NavigationToolbar

font = datavar.DefaultFont()
fontsmall = datavar.SmallFont()

## *******************************************
############## WIDGET ################
## *******************************************
class CCD(QtGui.QWidget):
    
    def __init__(self, parent=None):
        super(CCD, self).__init__()
        
        self.initUI()
        
    def initUI(self):               
        self.refstars, self.image, self.header, self.afrho, self.starlist, self.maglim, self.image_data, self.com_rade = [], [], [], {}, [], 0, [], []

        # Input grid
        grid = QtGui.QGridLayout()
        
        btnnew = QtGui.QPushButton('New observation', self) 
        btnnew.setFont(font)
        btnnew.clicked.connect(self.button_click_clear)
        
        labels = [ 'Comet name', 'Date YYYY/MM/DD', 'Time HH:MM (UT)', 'Filter', 'Mag. source', \
                   'Corrections', 'Binning', 'Nr. of frames', 'Total exp. time(s)', 'Coma diameter (\')',\
                   u'Tail length (\u00b0)', u'Position angle(\u00b0)' ]
        self.elements = []
        
        for i, name in enumerate(labels):
            label = QtGui.QLabel(name)
            label.setFont(font)
            grid.addWidget(label, i, 0 )
            if i in [ 3,4,5,6 ]:
              self.elements.append( QtGui.QComboBox(self) )
              self.elements[i].setStyleSheet("selection-color: black")
            elif i == 0: 
              self.elements.append( QtGui.QLineEdit(self) )
              self.elements[i].setDisabled(1)
            else:
              self.elements.append( QtGui.QLineEdit(self) )
            self.elements[i].setFont(font)
            self.elements[i].setSizePolicy(QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Maximum))
            grid.addWidget(self.elements[i], i, 1 )        

        # Settings grid values for combobox
        for item in datavar.Filters():
          self.elements[3].addItem(item[0])
        self.elements[3].setCurrentIndex(int(koprhand.SettingsRead()[1][10]))
        self.elements[3].activated.connect(self.onActivatedFlt)

        for item in datavar.MagSources():
          self.elements[4].addItem(item[0])

        for item in datavar.Corrections():
          self.elements[5].addItem(item[0])
        self.elements[5].setCurrentIndex(int(koprhand.SettingsRead()[1][9]))

        for item in ['1x1','2x2','3x3']:
          self.elements[6].addItem(item)

        self.measure = QtGui.QComboBox(self)
        for item in ['Measure Comet','Measure Tail','Measure Stars','Fix Astrometry']:
          self.measure.addItem(item)
        self.cbext = QtGui.QComboBox(self)
        for item in ['No Extinction','Winter','Spring/Autumn','Summer']:
          self.cbext.addItem(item)

        bedit = QtGui.QPushButton('Reference stars', self)
        bedit.clicked.connect(self.button_click_edit)
        bproc = QtGui.QPushButton('Process obs.', self)
        bproc.clicked.connect(self.button_click_calc)

        items = [ bedit, bproc ]
        for i, item in enumerate(items):
          item.setFont(font)
          grid.addWidget(item, i+12, 0 )

        items = [ self.measure, self.cbext]
        for i, item in enumerate(items):
          item.setFont(font)
          item.setStyleSheet("selection-color: black")
          grid.addWidget(item, i+12, 1 )

        self.comment = QtGui.QTextEdit(self)
        self.comment.setText('Comment')
        self.comment.setFont(font) 
        self.comment.setMaximumHeight(bproc.sizeHint().height() * 3)
        grid.addWidget(self.comment, 15, 0, 2, 2 )

        datavar.clickable(self.elements[1]).connect(self.datetime)
        datavar.clickable(self.elements[2]).connect(self.datetime)        
        
        # Info Grid
        self.infolabels = [ 'Dist.(AU)= ', 'r(AU)= ', u'Alt.(\u00b0)= ', u'Sun alt.(\u00b0)= ', u'Moon alt.(\u00b0)= ' ]
        self.info = []

        infogrid = QtGui.QGridLayout()
        for i, name in enumerate(self.infolabels):
            self.info.append(QtGui.QLabel(name))
            self.info[i].setFont(fontsmall)
            self.info[i].setMaximumHeight(bproc.sizeHint().height()*0.5)
            self.info[i].setMinimumWidth(bproc.sizeHint().width())
            infogrid.addWidget(self.info[i], 0, i )

        # Image control
        btnopen = QtGui.QPushButton('Open FITS Image', self)
        btnopen.setFont(font)
        btnopen.clicked.connect(self.button_click_img) 
        
        btnhead = QtGui.QPushButton('FITS Header', self)
        btnhead.setFont(font)
        btnhead.clicked.connect(self.fitsheader)
        
        self.centr = QtGui.QCheckBox('Centroid', self)
        self.centr.setFont(font) 
        self.centr.toggle()

        self.acf = QtGui.QCheckBox('ACF filter', self)
        self.acf.setFont(font) 
        self.acf.toggle()
        
        imglbl = [ 'Star mag. limit', 'Aperture radius', 'Saturation level' ]

        ctrlgrid = QtGui.QGridLayout()
        ctrlgrid.addWidget(btnopen, 0, 0, 1, 2)
        ctrlgrid.addWidget(self.centr, 0, 2)
        ctrlgrid.addWidget(self.acf, 0, 3)
        ctrlgrid.addWidget(btnhead, 0, 4, 1, 2)

        self.imgcont = []
        for i, name in enumerate(imglbl):
            label = QtGui.QLabel(name)
            label.setFont(font)
            ctrlgrid.addWidget(label, 1, i * 2 )
            self.imgcont.append(QtGui.QLineEdit(self))
            self.imgcont[i].setFont(font)
            self.imgcont[i].setMaximumWidth(self.centr.sizeHint().width())
            ctrlgrid.addWidget(self.imgcont[i], 1, i * 2 + 1 )

        datavar.clickable(self.imgcont[0]).connect(self.starredraw)
        datavar.clickable(self.imgcont[1]).connect(self.apertdraw)
        
        # Image field
        self.sc = koprfig.ImageDialog(self, image_data=[np.zeros((768,1024), dtype = np.uint16 ), 0, 1, 2 ]  )

        self.mpl_toolbar = NavigationToolbar(self.sc.canvas, self)

        vboxl = QtGui.QVBoxLayout() 
        vboxl.addWidget(btnnew)
        vboxl.addLayout(grid)
        vboxl.addStretch(1)

        vboxr = QtGui.QVBoxLayout() 
        hbox = QtGui.QHBoxLayout()
        hbox.addLayout(ctrlgrid)
        hbox.addStretch(1)
        vboxr.addLayout(hbox)
        vboxr.addWidget(self.mpl_toolbar)
        vboxr.addWidget(self.sc)
        vboxr.addLayout(infogrid)
        
        hbox = QtGui.QHBoxLayout()
        hbox.addLayout(vboxl)
        hbox.addLayout(vboxr)
        
        self.setLayout(hbox)            
        self.setWindowTitle('KOPR - CCD observation')    
        self.show()        

        # getting info about comet
    def InfoInit(self): 
        params = koprhand.SettingsRead()
        destig, date, time = str(self.elements[0].text()), str(self.elements[1].text()), str(self.elements[2].text()) 
        comet, moonalt, moonp, sunalt = koprfunc.CometObsInf(destig, date, time, params[2][1], params[2][2], params[2][3])
        cometinfo = [ str(round(comet.dist,3)), str(round(comet.r,3)), str(round(comet.alt,2)), str(sunalt), str(moonalt) ]
        self.com_rade = comephem.j2000rade( comet.ra, comet.de, date, time ) # ra de for comet pointer
        for i in range(len(cometinfo)):
          self.info[i].setText(self.infolabels[i]+cometinfo[i])

        # listening date and time fields 
    def datetime(self):
        if self.elements[1].text() and self.elements[2].text():
          date = str(self.elements[1].text())
          time = str(self.elements[2].text())
          
          if len(date.split('/')) == 3 and len(time.split(':')) == 2:
            destig = str(self.elements[0].text())

            if len(date.split('/')[1]) == 2 and len(time.split(':')[1]) == 2 and destig != '':
               check = True
               for num in date.split('/'):
                 if not num.isdigit() or len(num) not in (4,2):
                   check = False
                   break
               for num in time.split(':'):
                 if not num.isdigit() or len(num) <> 2:
                   check = False
                   break

               if check:
                 self.InfoInit()

    def starredraw(self):
        if self.header and self.imgcont[0].text():
          maglim = self.imgcont[0].text()
          magsrc = self.elements[4].currentIndex()
          if float(maglim) > 1 and maglim != self.maglim:
            try:
#              self.starlist, blustar, yelstar, redstar, nonestar = koprfunc.VizierStars(self.header, float(maglim), magsrc)
              self.starlist, blustar, yelstar, redstar, nonestar = koprfunc.StarData(self.header, float(maglim), magsrc)
              self.sc.bluplt.set_xdata([item[0] for item in blustar])
              self.sc.bluplt.set_ydata([item[1] for item in blustar])
              self.sc.yelplt.set_xdata([item[0] for item in yelstar])
              self.sc.yelplt.set_ydata([item[1] for item in yelstar])
              self.sc.redplt.set_xdata([item[0] for item in redstar])
              self.sc.redplt.set_ydata([item[1] for item in redstar])
              self.sc.graplt.set_xdata([item[0] for item in nonestar])
              self.sc.graplt.set_ydata([item[1] for item in nonestar])
              self.sc.canvas.draw_idle()
              self.maglim = maglim
            except ValueError:
              koprdial.InfoDialog(2,'FITS Header error','Cannot read image header, astrometry is not possible')
   
        # update value of image by smin smax
    def img_update(self, val):
        self.sc.img.set_clim([self.sc.smin.val,self.sc.smax.val])
        self.sc.canvas.draw_idle()
    
    def image_redraw(self):
        self.refstar = {}
        self.tail_lst = []
        self.astron_lst = [ [], [], [], [], 0 ]
        self.shift_is_held = False
      
        self.cid = self.sc.canvas.mpl_connect('button_press_event',self.onpickStar)
        self.cidsp = self.sc.canvas.mpl_connect('key_press_event', self.on_key_press)
        self.cidsr = self.sc.canvas.mpl_connect('key_release_event', self.on_key_release)
       
        # set up new image
        self.sky, satur, datamin, datamax, median = koprfunc.ImageStats(self.image_data)
        self.imgcont[2].setText(str(satur)) 

        self.sc.AxRefresh( [self.image_data, datamin, datamax, median] )

        self.sc.axmin.clear()
        self.sc.axmax.clear()
        self.sc.smin.__init__(self.sc.axmin, 'Min', 0, median, valinit=datamin)
        self.sc.smax.__init__(self.sc.axmax, 'Max', median, datamax*5., valinit=datamax)

        self.sc.smin.on_changed(self.img_update)
        self.sc.smax.on_changed(self.img_update)

        # ------------- Comet poiner addon START
        params = koprhand.SettingsRead()
        if params[1][11] == '2':
          comx, comy = koprfunc.WorldtoPix( self.header, self.com_rade[0]*15., self.com_rade[1] )
          self.sc.com_plot.set_xdata(comx)
          self.sc.com_plot.set_ydata(comy)
        # ------------- Comet poiner addon END
        self.maglim = 1
        self.starredraw()
        
        self.sc.canvas.draw_idle()

    def button_click_img(self):
        # load image first
        fname, self.image_data, shape, self.header = koprhand.ImageLoad(self.image_data, 0, self.header)

        xasr, yasr = self.SetBinning() # find image binning
        if xasr != yasr:
          koprdial.InfoDialog(1, 'Image aspect ratio error', 'Aspect ratio of image doesn\'t match your selected CCD chip.')
        elif fname:
            # check image header for valies
            if 'EXPTIME' in self.header:
                self.elements[8].setText( str( round( float(self.header['EXPTIME']),1) ) )
            else:
                text, ok = QtGui.QInputDialog.getText(self, 'Exptime not found', 'Enter exposure time:')
                if ok and str(text).isdigit():
                  self.elements[8].setText( str( round( float(text),1) ) )
                else:
                  koprdial.InfoDialog(1, 'Exposure time error', 'Enter exposure time in seconds!') 

            if 'DATE-OBS' in self.header:
                obsdate, obstime = self.header['DATE-OBS'].split('T')
                self.elements[1].setText(obsdate.replace('-','/'))
                self.elements[2].setText(obstime.rsplit(':',1)[0])
                
            if 'FILTER' in self.header:
                flt = str(self.header['FILTER']).strip().replace('N','C')
                for item in datavar.Filters():
                  if item[0].split('-')[1].strip()[0:1] == flt[0:1]:
                    self.elements[3].setCurrentIndex(item[1])
                    if item[1] == 0:                    
                      self.elements[4].setCurrentIndex(1)
                    else:
                      self.elements[4].setCurrentIndex(item[1]-1)
            else:
                flt = 'C'

            try:   
              params = koprhand.SettingsRead()           
              if flt == 'C' and params[5][8].isdigit():
                self.elements[4].setCurrentIndex(datavar.retsrc(params[5][8]))
            except:
              pass

            if 'SUBFRMS' in self.header:
                self.elements[7].setText(str(self.header['SUBFRMS']).strip())
                      
            # check if comet name is set
            if not self.elements[0].text():
              text, ok = QtGui.QInputDialog.getText(self, 'Comet destignation', 'Comet short name (e.g. 103P or C/2012 X1): ')
              if text: 
                  eline = koprhand.loadcometid(text)
                  if eline == '':
                      koprdial.InfoDialog(1, 'Comet error', 'Elements not found! Add this comet first.')
                  else:    
                      self.elements[0].setText(text)

            self.datetime() # refresh comet info
            
            self.image_redraw() # draw image
        
        # observation reset
    def button_click_clear(self):
        self.elements[0].clear()
        for i in [2,7,8,9,10,11]:
          self.elements[i].clear()
        
        for i in [3,4,6]:
          self.elements[i].setCurrentIndex(0)
        
        self.imgcont[0].clear()
        self.measure.setCurrentIndex(0)
        self.cbext.setCurrentIndex(0)

        self.comment.setText('Comment')
        self.refstars, self.image, self.header, self.afrho, self.starlist, self.maglim, self.image_data, self.com_rade = [], [], [], {}, [], 0, [], []
        self.ccoords = []
        self.sc.starcirc = {}
        params = koprhand.SettingsRead()
        text, ok = QtGui.QInputDialog.getText(self, 'New CCD/DSLR observation', 'Measure for location: '+params[2][0]+'\nTelescope: '+params[3][0]+'\nCamera: '+params[5][0]+'\n\nInput comet short name (e.g. 103P or C/2012 X1):')                
        if text: 
            eline = koprhand.loadcometid(text)
            if eline == '':
                koprdial.InfoDialog(1, 'Comet error', 'Elements not found! Add this comet first.')
            else:    
                self.elements[0].setText(text)
                self.button_click_img()

    def fitsheader(self):
        if self.header:
          a = koprdial.MagDataOutput( repr(self.header), 'Fits header' )
          a.exec_()
        else: 
          koprdial.InfoDialog(1, 'Errors were encountered', 'Load image first!')

    def button_click_calc(self):
        params = koprhand.SettingsRead()
        code = koprclass.icqcode()
        locname, lon, lat, elev, gmt = params[2] 
        code.comet, code.date, code.time = str(self.elements[0].text()), str(self.elements[1].text()), str(self.elements[2].text())
        apert, tail, pa = str(self.elements[9].text()), str(self.elements[10].text()), str(self.elements[11].text())

        critical, errcount, errormsg, errordetmsg = koprhand.showdialogCCD( self.refstars, apert, tail, pa, self.elements[8].text(), self.elements[7].text() )

        if not all(star[6] == self.refstars[0][6] for star in self.refstars):
            koprdial.InfoDialog(1, 'Magnitude source mismatch', 'There are objects with different magnitude source.')

        elif critical == 1:
            koprdial.InfoDialog(1, 'Errors were encountered', str(errcount) + ' errors occured: ', errormsg, errordetmsg)
        else:        
            comet, moonalt, moonp, sunalt = koprfunc.CometObsInf(code.comet, code.date, code.time, lon, lat, elev)
            code.ttype, code.apert, tfocl = params[3][1], params[3][2], params[3][3] 
            ccdcam, code.cam, chip, code.chip, xres, yres, xpix, ypix, peak = params[5]
            xres, yres, xpix, ypix, code.apert, tfocl = float(xres),float(yres),float(xpix),float(ypix), float(code.apert), float(tfocl)
            code.otype, code.sfw = 'ccd', 'KPR' 
            
            flt = str(self.elements[3].currentText()).split('-')[1].strip()
            code.method, code.magsrc = datavar.MagMethods(flt)

            code.focl, code.obscode, code.corr = tfocl/code.apert, params[0][0] ,self.elements[5].currentIndex()   
                    
            # Nalezeni komety a kalkulace
            c, moonalt, moonp, sunalt = koprfunc.CometObsInf(code.comet, code.date, code.time, str(lon), str(lat), elev)
            comet, cometalt, rdist, ddist, phasea = c.name, round(c.alt,2), round(c.r,3), round(c.dist,3), c.phase
            extinction = str(self.cbext.currentText())
            
            for star in self.refstars:
                if 'c' in star[0]:
                    cometrel = star[1]
                    comalt = koprhand.degconvert(str(star[4]))

            code.mag, code.error, xlist = koprfunc.CalculateMag( [cometrel, comalt], self.refstars, extinction, elev)

            code.xpix, code.ypix = round(xpix/tfocl*206.3,1)*(self.elements[6].currentIndex()+1), round(ypix/tfocl*206.3,1)*(self.elements[6].currentIndex()+1)
            code.coma = float(self.elements[9].text()) ##### Coma input
            if str(self.elements[10].text()) <> '':
                code.tail, code.tailpa = koprhand.degconvert(str(self.elements[10].text())), int(float(self.elements[11].text())) 
            code.zoom, code.fnumb = int(float(self.elements[8].text())), int(self.elements[7].text())
            # generovani ICQ kodu
            code.obsnote = 'Comet Alt.:'+str(round(cometalt,1))+' Calculated mags:'+','.join(str(round(x,1)) for x in xlist)+'\n' 
            if str(self.comment.toPlainText()) != '' and str(self.comment.toPlainText()) != 'Comment':
                code.obsnote = str(self.comment.toPlainText()) + code.obsnote

            output, output2 = code.build_icq()
            
            # vybudovat x, y afrho list
            # pixszdeg = (round(xpix/tfocl*206.3,1) + round(ypix/tfocl*206.3,1)) / 7200. 
            try:
              pixszdeg = abs(float(self.header['CDELT1']))
            except:
              pixszdeg = (round(xpix/tfocl*206.3,1) + round(ypix/tfocl*206.3,1)) / 7200. 

            afrho, afoutpt, limita = koprfunc.AfrhoCalc( self.afrho, self.refstars, extinction, elev, pixszdeg, ddist, rdist, comalt, phasea, code, flt, int(params[1][3]) )

            print limita
            if limita < -0.15:
               koprdial.InfoDialog(0, 'Increase aperture', 'Your measure is most likely underestimated, please remeasure comet with larger aperture')           
          
            # vytvorit vystup
            plot = [ afoutpt[0], afoutpt[1], afoutpt[0], afoutpt[3], -100, code.comet+' '+code.date + ' ' + code.time+' '+code.obscode, afoutpt[4], afoutpt[5] ]
            
            critical, warning, errcount, errormsg, errordetmsg = koprhand.showdialogOut( cometalt, code.mag, moonalt, moonp, sunalt, 1, code.error, str(self.cbext.currentText()), []) #output errors
            if critical == 1:
                koprdial.InfoDialog(1, 'Errors were encountered', str(errcount) + ' errors occured: ', errormsg, errordetmsg)
            else:
                if warning == 1:
                    koprdial.InfoDialog(2, 'There are warnings', str(errcount) + ' warnings occured: ', errormsg, errordetmsg)
                a = koprdial.ObsOutput( output, output2 + ' ({:s} {:s}, {:s})'.format( params[0][1], params[0][2], locname ), code.obscode, plot, afrho = afrho )
                a.exec_()

    def button_click_edit(self):
        params = koprhand.SettingsRead()
        if not all(star[6] == self.refstars[0][6] for star in self.refstars):
            koprdial.InfoDialog(1, 'Magnitude source mismatch', 'There are objects with different magnitude source.')
        else:
            a = koprdial.StarDialog(typ = 'ccd', refno = len(self.refstars), destig = str(self.elements[0].text()), date = str(self.elements[1].text()), time = str(self.elements[2].text()),\
                                 extinct = self.cbext.currentIndex() ,lon = params[2][1], lat = params[2][2], elev = params[2][3], stars = self.refstars )             
            a.exec_()
            if a.output == [] and self.refstars != []:
                for key in self.refstar.keys():
                    self.sc.starcirc[key].remove() # delete if duplicate
                    self.sc.starcirc[(key,'i')].remove() # delete if duplicate
                    self.sc.starcirc[(key,'o')].remove() # delete if duplicate
                    del self.sc.starcirc[key] # delete if duplicate
                    del self.sc.starcirc[(key,'i')] # delete if duplicate
                    del self.sc.starcirc[(key,'o')] # delete if duplicate
                    del self.refstar[key]
                self.sc.canvas.draw_idle()

            self.cbext.setCurrentIndex(a.extinct)
            self.refstars = a.output

        # on filter change
    def onActivatedFlt(self):
        if self.elements[3].currentIndex() != 0:
           self.elements[4].setCurrentIndex(self.elements[3].currentIndex()-1)

        # find binning of image
    def SetBinning(self):
        params = koprhand.SettingsRead()
        imgy, imgx = self.image_data.shape

        xres, yres = float(params[5][4]), float(params[5][5])
        if imgy < imgx and int(xres) > int(yres):
          xasr, yasr = xres/imgx, yres/imgy
        else:
          yasr, xasr  = xres/imgx, yres/imgy

        if int( round(xasr)) in (1,2,3):
          self.elements[6].setCurrentIndex( int( round(xasr)) - 1 )
        else:
          self.elements[6].setCurrentIndex( 0 )

        return int( round(xasr)), int( round(yasr)) 

    def on_key_press(self, event):
       if event.key == 'shift':
           self.shift_is_held = True
    
    def on_key_release(self, event):
       if event.key == 'shift':
           self.shift_is_held = False

    def tailredraw(self, xdata, ydata):
       self.tail_lst.append( (xdata, ydata) )
       x, y = zip(*self.tail_lst)
       self.sc.tpt_plot.set_xdata(x)
       self.sc.tpt_plot.set_ydata(y)
       self.sc.canvas.draw_idle()

    def realaredraw(self, xdata, ydata):
       self.astron_lst[0].append( (xdata, ydata) )
       x, y = zip(*self.astron_lst[0])
       self.sc.real_plot.set_xdata(x)
       self.sc.real_plot.set_ydata(y)
       self.sc.canvas.draw_idle()
           
    def maparedraw(self, xdata, ydata):
       self.astron_lst[1].append( (xdata, ydata) )
       x, y = zip(*self.astron_lst[1])
       self.sc.mapa_plot.set_xdata(x)
       self.sc.mapa_plot.set_ydata(y)
       self.sc.canvas.draw_idle()

    def mapaclear(self):
       self.sc.real_plot.set_xdata([])
       self.sc.real_plot.set_ydata([])
       self.sc.mapa_plot.set_xdata([])
       self.sc.mapa_plot.set_ydata([])
       self.sc.canvas.draw_idle()

    def aftercomet(self):
      self.a = CCDMeasure('comet')
      self.a.element[0].clicked.connect(self.button_measstar)
      self.a.element[1].clicked.connect(self.button_openstar)
      self.a.element[2].clicked.connect(self.button_meastail)
      self.a.element[3].clicked.connect(self.button_meascom)
      self.a.exec_()
      # measure tail ? measure stars ? open stars image? measure comet again?

    def aftertail(self):
      self.a = CCDMeasure('tail')
      self.a.element[0].clicked.connect(self.button_measstar)
      self.a.element[1].clicked.connect(self.button_openstar)
      self.a.element[2].clicked.connect(self.button_meastail)
      self.a.exec_()
      # measure stars ? open stars image? measure tail again?

    def button_measstar(self):
      self.measure.setCurrentIndex(2)
      self.a.close()

    def button_openstar(self):
      self.measure.setCurrentIndex(2)
      self.a.close()
      self.button_click_img()

    def button_meastail(self):
      self.measure.setCurrentIndex(1)
      self.a.close()

    def button_meascom(self):
      self.measure.setCurrentIndex(0)
      self.a.close()

    def PerformCentroid(self, x, y):
      if self.centr.isChecked(): # centroid on
        return koprfunc.Centroid( self.image_data, x, y, self.sky, 18/(self.elements[6].currentIndex()+1) )
      else:
        return x, y

    def onpickStar(self, event): 
      if event.xdata and event.ydata:
        if self.measure.currentIndex() == 1 and event.button == 3: #tail input
            if len(self.tail_lst) == 0:
                xy = koprfunc.Centroid( self.image_data, event.xdata, event.ydata, self.sky )
                self.tailredraw( xy[0],xy[1] )
            elif len(self.tail_lst) == 1:  
                self.tailredraw(event.xdata, event.ydata)
                tail, pa = koprfunc.CalcTailpa(self.header, self.tail_lst[0][0], self.tail_lst[0][1], event.xdata, event.ydata)
                self.elements[10].setText( str(round(tail,3)) )
                self.elements[11].setText( str(round(pa,0)) )
                self.aftertail()
            elif len(self.tail_lst) == 2:
                self.tail_lst = []
                xy = koprfunc.Centroid( self.image_data, event.xdata, event.ydata, self.sky, 18/(self.elements[6].currentIndex()+1) )
                self.tailredraw( xy[0],xy[1] )                
        
        elif not self.imgcont[1].text() and event.button == 3: #not aperture
            koprdial.InfoDialog(1, 'Aperture error', 'Insert aperture radius in px first!')
        
        elif self.measure.currentIndex() == 3 and event.button == 3: #astrometry fix
            if self.astron_lst[4] in (0,2,4):
              xy = koprfunc.Centroid( self.image_data, event.xdata, event.ydata, self.sky, 18/(self.elements[6].currentIndex()+1) )
              self.realaredraw( xy[0],xy[1] )
              self.astron_lst[4] += 1
            elif self.astron_lst[4] in (1,3):
              star = koprfunc.ClosestStar( event.xdata, event.ydata, self.starlist )
              x, y = star[0], star[1]
              self.maparedraw( x, y )
              self.astron_lst[4] += 1
            elif self.astron_lst[4] == 5:
              star = koprfunc.ClosestStar( event.xdata, event.ydata, self.starlist )
              x, y = star[0], star[1]
              self.astron_lst[1].append( (x, y) )
              total = [ 0,0 ]
              for i in range(3):
                x1,y1 = self.astron_lst[0][i]
                x2,y2 = self.astron_lst[1][i]
                total[0] += x1 - x2
                total[1] += y1 - y2
              
              self.header['CRPIX1'] += total[0]/3.
              self.header['CRPIX2'] += total[1]/3.
              self.maglim = 0   

              self.starlist = []
              self.starredraw()
              
              self.mapaclear()
              self.astron_lst = [ [], [], [], [], 0 ]
              
              self.sc.canvas.draw_idle()
 
        elif event.button == 3: #aperture measures
            params = koprhand.SettingsRead()
            self.limit = float(self.imgcont[2].text()) # get values
            lon, lat, elev = float(params[2][1]), float(params[2][2]), float(params[2][3])
            destig, date, time = str(self.elements[0].text()), str(self.elements[1].text()), str(self.elements[2].text())

            ylim, xlim = self.image_data.shape
            results, oldresult, olddiff = {}, 0, 0           
            rad = self.GetRad(str(self.imgcont[1].text()))

            if int(event.xdata-(18/(self.elements[6].currentIndex()+1))-1) < 0 or int(event.xdata+(18/(self.elements[6].currentIndex()+1))+2) > xlim or int(event.ydata-(18/(self.elements[6].currentIndex()+1))-1) < 0 or int(event.ydata+(18/(self.elements[6].currentIndex()+1))+2) > ylim:
                koprdial.InfoDialog(1, 'Aperure out of image', 'Aperture is larger than image.')
            else:    
                if self.measure.currentIndex() == 0: #### comet position data ############
                    # draw aperture first
                    self.CometApertureRedraw(event.xdata, event.ydata, rad)
                    self.ccoords = [event.xdata, event.ydata]
                    # perform comet measure?
                    perform = QtGui.QMessageBox.question(self, 'Message', "Perform comet measure?", QtGui.QMessageBox.Yes | QtGui.QMessageBox.No, QtGui.QMessageBox.Yes)
                    if perform == QtGui.QMessageBox.Yes:
                      self.PerformCometMeasure(self.ccoords[0], self.ccoords[1], self.GetRad(str(self.imgcont[1].text())))

                elif self.measure.currentIndex() == 2: #### star measure ############
                  x, y = self.PerformCentroid(event.xdata, event.ydata)
                  image_data = self.image_data
                  sky, rectin, rectout = koprfunc.BackgroundSky( image_data, x, y, rad)
                  result, error = photom.Photometry( self.image_data, x, y, rad, sky, self.limit )

                  if error == 1: # saturated
                    koprdial.InfoDialog(1, 'Saturation possibility', 'A maximum value was reached in selected aperture. Use different star or change saturation limit for this image.')
                  else: 
                    if self.shift_is_held: # remove star 
                        distances, outlist = [], {} 
                        for key in self.refstar.keys():#find closest star to remove
                          if ((x-50) < key[0] < (x+50)) and ((y-50) < key[1] < (y+50)): #replace 50 with rad???
                            dx, dy = key[0]-x, key[1]-y
                            c = dx*dx + dy*dy
                            distances.append(c) 
                            outlist[c] = key                                 
                        if outlist:  
                            try:
                              self.sc.starcirc[outlist[min(distances)]].remove()
                              self.sc.starcirc[(outlist[min(distances)],'i')].remove()
                              self.sc.starcirc[(outlist[min(distances)],'o')].remove()
                              del self.sc.starcirc[outlist[min(distances)]]
                              del self.sc.starcirc[(outlist[min(distances)],'i')]
                              del self.sc.starcirc[(outlist[min(distances)],'o')]
                            except ValueError:
                              pass

                            self.sc.canvas.draw_idle()
                            for star in self.refstars:
                              if star[1] == self.refstar[outlist[min(distances)]][1] and star[2] == self.refstar[outlist[min(distances)]][2]:
                                self.refstars.remove(star)
                                break    

                            try:
                              del self.refstar[outlist[min(distances)]]
                            except KeyError:
                              pass
    
                    elif (x,y) not in self.refstar: #add star
                      try:
                        dist1, dist2 = 2*(rad/3. + rad)+1, 2*(rad/3. + rad*2)+1
                        
                        star = koprfunc.ClosestStar( event.xdata, event.ydata, self.starlist )

                        world = koprfunc.PixToWorld(self.header, x, y)
                        ra, de = float(world[0])/15., float(world[1])
                        starmag, bvind = star[2], star[3]
                        ident = str(star[0])+','+str(star[1]) 
    
                        # check for identical refstars
                        for item in self.refstars: 
                          if ident in item:
                            self.refstars.remove(item) # delete if duplicate

                        for key in self.refstar.keys():
                          if ident in self.refstar[key]:
                            del self.refstar[key]   # delete if duplicate
                            self.sc.starcirc[key].remove() # delete if duplicate
                            self.sc.starcirc[(key,'i')].remove() # delete if duplicate
                            self.sc.starcirc[(key,'o')].remove() # delete if duplicate
                            del self.sc.starcirc[key] # delete if duplicate
                            del self.sc.starcirc[(key,'i')] # delete if duplicate
                            del self.sc.starcirc[(key,'o')] # delete if duplicate

                            self.sc.canvas.draw_idle()
                        # end duplicate remove    

                        staralt = round(comephem.StarAlt( ra, de, date, time, lon, lat),2)

                        self.sc.starcirc[(int(x), int(y))] = plt.Circle((x, y), rad, color='c', fill = False, lw = 2 )
                        self.sc.starcirc[((int(x), int(y)), 'i')] = plt.Rectangle(rectin, dist1, dist1, color='c', fill = False)
                        self.sc.starcirc[((int(x), int(y)), 'o')] = plt.Rectangle(rectout, dist2, dist2, color='c', fill = False)
                        
                        self.sc.ax.add_artist(self.sc.starcirc[(int(x), int(y))])
                        self.sc.ax.add_artist(self.sc.starcirc[((int(x), int(y)), 'i')])
                        self.sc.ax.add_artist(self.sc.starcirc[((int(x), int(y)), 'o')])
                        
                        self.sc.canvas.draw_idle()

                        try:
                          self.refstar[(int(x), int(y))] = ['o', round(result, 4), starmag, str(round(bvind,1)), str(staralt), str(ra*15)+','+str(de), str(self.elements[4].currentText()), ident ]
                        except TypeError:
                          self.refstar[(int(x), int(y))] = ['o', round(result, 4), starmag, bvind, str(staralt), str(ra*15)+','+str(de), str(self.elements[4].currentText()), ident ]
 
                        self.refstars.append(self.refstar[(int(x), int(y))])

                      except ValueError:
                        #self.refstar[(int(x), int(y))] = ['x', round(result, 4), 0, '', '', '', str(self.elements[4].currentText()) ]
                        koprdial.InfoDialog(0, 'No star found', 'No star found or no APASS data available for this star')
                      #### star measure end ############
    def GetRad(self, rad):
      if 'kkm' in rad: # aperture from kkm conversion
        destig, date, time = str(self.elements[0].text()), str(self.elements[1].text()), str(self.elements[2].text())
        rad = koprfunc.KkmToPx(rad, self.header['CDELT1'], destig, date, time)
        self.imgcont[1].setText(str(rad))
        return rad
      else:
        return int(rad) # aperture radius

    def apertdraw(self): # redraw when aperture increase
      if self.ccoords and self.measure.currentIndex() == 0:
        self.CometApertureRedraw(self.ccoords[0], self.ccoords[1], self.GetRad(str(self.imgcont[1].text())))

    def CometApertureRedraw(self, x, y, rad): 
                    image_data = self.image_data           
                    sky, rectin, rectout = koprfunc.BackgroundSky(image_data, x, y, rad) 

                    if 'comet' in self.sc.starcirc: # if exist, remove old comet
                      try:
                        self.sc.starcirc['comet'].remove()
                        self.sc.starcirc['cometi'].remove()
                        self.sc.starcirc['cometo'].remove()
                        del self.sc.starcirc['comet']
                        del self.sc.starcirc['cometi']
                        del self.sc.starcirc['cometo']
                      except ValueError:
                        pass

                      for star in self.refstars:
                        if star[0] == 'c':
                          self.refstars.remove(star)
                          break    

                      try:
                        del self.refstar['comet']
                      except KeyError:
                        pass

                    dist1, dist2 = 2*(rad/3. + rad)+1, 2*(rad/3. + rad*2)+1
                    self.sc.starcirc['comet'] = plt.Circle((x, y), rad, color='g', fill = False, lw=2)
                    self.sc.starcirc['cometi'] = plt.Rectangle(rectin, dist1, dist1, color='g', fill = False)
                    self.sc.starcirc['cometo'] = plt.Rectangle(rectout, dist2, dist2, color='g', fill = False)
                    self.sc.ax.add_artist(self.sc.starcirc['comet'])#print 'comet'
                    self.sc.ax.add_artist(self.sc.starcirc['cometi'])#print 'comet'
                    self.sc.ax.add_artist(self.sc.starcirc['cometo'])#print 'comet'
                    self.sc.canvas.draw_idle()

    def PerformCometMeasure(self, x, y, rad):
            params = koprhand.SettingsRead()
            self.limit = float(self.imgcont[2].text()) # get values
            lon, lat, elev = float(params[2][1]), float(params[2][2]), float(params[2][3])
            destig, date, time = str(self.elements[0].text()), str(self.elements[1].text()), str(self.elements[2].text())

            ylim, xlim = self.image_data.shape
            results, oldresult, olddiff = {}, 0, 0           
            image_data = self.image_data
            self.afrho = {}
            x, y = self.PerformCentroid(x, y)

            if int(x-rad-1) < 0 or int(x+rad+2) > xlim or int(y-rad-1) < 0 or int(y+rad+2) > ylim:
              koprdial.InfoDialog(1, 'Aperure out of image', 'Aperture is larger than image.')
            else:    
                  sky, rectin, rectout = koprfunc.BackgroundSky(image_data, x, y, rad) 
                  print 'Background sky value ',sky

                  if self.acf.isChecked():#median coma filter
                      result, error, self.afrho = photom.PhotometryACF( self.image_data, x, y, rad, sky, self.limit, params[1] )
                  else:
                      i = int(params[1][4])
                      while i < rad:
                        result, error = photom.Photometry( self.image_data, x, y, i, sky, self.limit )
                        self.afrho[i] = ( result, error )
                        i += int(params[1][5])
                      result, error = photom.Photometry( self.image_data, x, y, rad, sky, self.limit )
                      self.afrho[i] = ( result, error )
                    
                  if error == 1: # saturated
                    koprdial.InfoDialog(1, 'Saturation possibility', 'A maximum value was reached in selected aperture. Use different star or change saturation limit for this image.')
                  else:
                    c = koprfunc.CometObsInf(destig, date, time, lon, lat, elev)[0]
                    self.refstar['comet'] = ['c', round(result, 4 ), 0, '', str(round(c.alt, 2)), '', str(self.elements[4].currentText())]
                    
                    for star in self.refstars:
                      if star[0] == 'c':
                        self.refstars.remove(star)
                        break    

                    self.refstars.append(self.refstar['comet']) 

                    xpix, ypix, tfocl = float(params[5][6]), float(params[5][7]), float(params[3][3]) 
                    print tfocl
                    xpix, ypix = round(xpix/tfocl*206.3,1)*(self.elements[6].currentIndex()+1), round(ypix/tfocl*206.3,1)*(self.elements[6].currentIndex()+1)
                    self.elements[9].setText( str( round( (rad * (xpix+ypix)/2.)/60.*2.,2) ) ) 
                    self.aftercomet()

## *******************************************
############## Dialog classes ################
## *******************************************

class CCDMeasure(QtGui.QDialog):
    def __init__(self, typ, parent = None):
        super(CCDMeasure, self).__init__()
        vbox = QtGui.QVBoxLayout()
        self.element = {} 
        
        grid = QtGui.QGridLayout()
        names = [ 'Measure Stars', 'Open star image', 'Measure Tail', 'Measure Comet Again', 'Measure Tail Again' ]
        if typ == 'comet':
          for i in range(4):
            self.element[i] = QtGui.QPushButton(names[i], self)
            grid.addWidget(self.element[i], *(i,0))
        elif typ == 'tail':
          names.remove('Measure Tail')
          names.remove('Measure Comet Again')
          for i in range(3):
            self.element[i] = QtGui.QPushButton(names[i], self)
            self.element[i].setFont(font)
            grid.addWidget(self.element[i], *(i,0))
        self.setLayout(grid)  
        self.setWindowTitle('Measure choices')
        self.setWindowModality(QtCore.Qt.ApplicationModal)      

class ScrollCCD(QtGui.QScrollArea):
    """ This scroll area only handles one widget for which scrolling should be provided. """
    def __init__(self, parent=None):
        QtGui.QScrollArea.__init__(self, parent)

        self.w = CCD(self)
        self.setWidget(self.w)    

