#!/usr/bin/python
# -*- coding: utf-8 -*-
#"""
#KOPR 1.0
#"""

import sys
import copy
import koprhand
import koprfunc
import koprclass
import koprdial
import koprfig
import datavar
import npfunc
import datetime
import setupobs
from PyQt4 import QtCore, QtGui
#import matplotlib
#matplotlib.use('Qt4Agg')
import matplotlib.pyplot as plt
from matplotlib.backends.backend_qt4agg import NavigationToolbar2QT as NavigationToolbar
from os import listdir
from os.path import isfile, join

font = datavar.DefaultFont()

## *******************************************
############## WIDGET ################
## *******************************************
class Analyzer(QtGui.QWidget):
    
    def __init__(self, parent=None):
        super(Analyzer, self).__init__()
        self.initUI()
        
    def initUI(self):               
        self.comets = []
        self.settings = [ 0, 0, 0, 1., 0, 0, 0, 0, 0, 4. ] # default settings
        self.peridict = {} # dictionary of returns
        self.returns = [] # list of returns containing [ year, on/off, color ]
        self.types = [['ccd', 1, 'o,circle'], ['vis', 1, 's,square'], ['mpct', 1, 'v,triangle_down'], ['mpcn', 1, '^,triangle_up']]
        self.obscode = [] # list of observers containing [ obscode:, on/off ]
        self.obs_list = [] # observations array
        self.working_list = [] # observations working list array

        self.items = [f.split('.')[0] for f in listdir('./comet-data/') if isfile(join('./comet-data/', f))]
        
        #btnnew.clicked.connect(self.button_click_clear)
        grid = QtGui.QGridLayout()
        self.listEnabled = QtGui.QListWidget()
        self.listEnabled.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
        
        self.listDisabled = QtGui.QListWidget()
        self.listDisabled.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)

        self.btnexcl = QtGui.QPushButton('Exclude >>', self)
        self.btnexcl.clicked.connect(self.button_click_disable)

        self.btnincl = QtGui.QPushButton('<< Include', self)
        self.btnincl.clicked.connect(self.button_click_enable)

        for i, line in enumerate([ [QtGui.QLabel('Included'), QtGui.QLabel('Excluded')] ,[self.listEnabled, self.listDisabled], [self.btnexcl, self.btnincl] ]):
          for j, element in enumerate(line):
            element.setFont(font)
            element.setMaximumWidth(100) 
            grid.addWidget( element, i, j )

        for item in self.items:
           self.listDisabled.addItem(item)

        btnset = QtGui.QPushButton('Settings and selections', self)
        btnset.setFont(font)
        btnset.clicked.connect(self.button_click_setup)
        
        vboxl = QtGui.QVBoxLayout()
        vboxl.addLayout(grid)
        vboxl.addWidget(btnset)
        vboxl.addStretch(1)

        plotgrid = QtGui.QGridLayout()
        self.plotopt = [ [ QtGui.QLabel('Axis x ranges'), QtGui.QLineEdit(self), QtGui.QLineEdit(self), QtGui.QComboBox(self), QtGui.QCheckBox('Compare', self) ],
                         [ QtGui.QLabel('Axis y ranges'), QtGui.QLineEdit(self), QtGui.QLineEdit(self), QtGui.QComboBox(self), QtGui.QPushButton('Refresh', self) ] ]
        
        for i, line in enumerate(self.plotopt):
          for j, item in enumerate(self.plotopt[i]):
            self.plotopt[i][j].setFont(font)
            if j == 3:
              self.plotopt[i][j].setStyleSheet("selection-color: black")
            plotgrid.addWidget(self.plotopt[i][j], i, j) 

        for i, item in enumerate( ['Date','Days from T','r','Log(r)'] ):
          self.plotopt[0][3].addItem(item)
        self.plotopt[0][3].activated[str].connect(self.onActivatedDateR)
        self.dater = self.plotopt[0][3].currentIndex() 
        
        for i, item in enumerate( ['mag','mag - 5*log(dist)'] ):
          self.plotopt[1][3].addItem(item)
        self.plotopt[1][3].activated[str].connect(self.onActivatedMagR)
        
        self.plotopt[1][4].clicked.connect(self.button_click_update)
        
        self.sc = koprfig.LightCurve(self)
        self.sc.ax.grid(True)
        self.mpl_toolbar = NavigationToolbar(self.sc.canvas, self)

        vboxr = QtGui.QVBoxLayout()
        hbox = QtGui.QHBoxLayout()
        hbox.addLayout(plotgrid)
        hbox.addStretch(1)
        vboxr.addLayout(hbox)
        vboxr.addWidget(self.mpl_toolbar)
        vboxr.addWidget(self.sc)

        hbox = QtGui.QHBoxLayout()
        hbox.addLayout(vboxl)
        hbox.addLayout(vboxr) 
        
        self.setLayout(hbox)            
        self.setWindowTitle('KOPR - Comparison')    
        self.show()        

### ************ NEW FUNCTIONS *********************
    def button_click_disable(self):
        rows = sorted([index.row() for index in self.listEnabled.selectedIndexes()], reverse=True)
        for row in rows:
          self.listDisabled.addItem(self.listEnabled.takeItem(row))
        self.listDisabled.sortItems()
        self.UpdComets()
        
    def button_click_enable(self):
        rows = sorted([index.row() for index in self.listDisabled.selectedIndexes()], reverse=True)       
        for row in rows:
          self.listEnabled.addItem(self.listDisabled.takeItem(row))
        self.listEnabled.sortItems()
        self.UpdComets()
        
    def UpdComets(self):
        self.comets = []
        for index in xrange(self.listEnabled.count()):
          self.comets.append( [ str(self.listEnabled.item(index).text()), [], [] ] )
        
        for i, comet in enumerate(self.comets): # for every comet load data
          try: 
            comet_file = open('./comet-data/'+comet[0]+'.dat', 'r') #nahrajeme pozorovani
            self.comets[i][1] = comet_file.readlines()
            comet_file.close()
            
            # for every data load perihelions
          except:
            pass        
        
        print self.comets
        
    def onActivatedDateR(self):
        ranges = [ str(self.plotopt[0][1].text()), str(self.plotopt[0][2].text()), self.dater, self.plotopt[0][3].currentIndex() ]
        
        if self.working_list:
          matrix = npfunc.RangeConversion(self.kometa, self.returns, self.peridict, self.working_list, ranges, int(self.plotopt[0][4].isChecked()))
          if self.plotopt[0][3].currentIndex() == 0:
            self.plotopt[0][1].setText( matrix[0][self.plotopt[0][3].currentIndex()] )
            self.plotopt[0][2].setText( matrix[1][self.plotopt[0][3].currentIndex()] )
          else:  
            self.plotopt[0][1].setText( str( round(matrix[0][self.plotopt[0][3].currentIndex()],3) ) )
            self.plotopt[0][2].setText( str( round(matrix[1][self.plotopt[0][3].currentIndex()],3) ) )
          self.button_click_update()
          
        else:
          self.plotopt[0][1].setText('')
          self.plotopt[0][2].setText('')
        
        self.dater = self.plotopt[0][3].currentIndex()

    def onActivatedMagR(self):
        if self.working_list:
          self.button_click_update()

    def button_click_setup(self):
        tabdialog = setupobs.TabDialog(self.settings, self.types, self.obscode, self.returns)
        tabdialog.exec_()
        self.settings = tabdialog.settings
        self.types = tabdialog.cat
        self.returns = tabdialog.returns
        self.obscode = tabdialog.obscode

    def button_click_update(self):
        elements, cobs_id, cobs_number = koprhand.loadcometid( str(self.comlbl.text()) )
        self.kometa = koprclass.comet()
        self.kometa.LoadComet(str(self.comlbl.text()))
        self.UpdatePlot(cobs_id, False)
    
    def button_click_clear(self):
        text, ok = QtGui.QInputDialog.getText(self, 'Comet destignation', 'Comet short name (e.g. 103P or C/2012 X1): ')                
        if text: 
            # updating list of COBS comets from internet and loading elements
            self.peridict, self.returns, self.obscode, self.obs_list, self.working_list = {}, [], [], [], []
            #koprhand.updatecometlist()
            elements, cobs_id, cobs_number = koprhand.loadcometid(text)
            if elements == '':
                koprdial.InfoDialog(1, 'Comet error', 'Elements not found! Add this comet first.')
            else:    
                # clean up fields first
                self.perihelions, self.observers, self.obs_list, self.peridict = [], [], [], {}
                self.plotopt[0][1].setText('')
                self.plotopt[0][2].setText('')
                self.plotopt[1][1].setText('')
                self.plotopt[1][2].setText('')
                
                self.reg.setText('')
                self.adit.setText('')

                self.sc.ax.clear()
                self.sc.ax.grid(True)
                self.sc.fig.tight_layout()  
                self.sc.canvas.draw()
                
                # Load comet data
                self.comlbl.setText(text) # set new comet text
                self.kometa = koprclass.comet()
                self.kometa.LoadComet(str(text))
                output = ''
                for line in self.kometa.magparams:
                    destig, h0, n, strrange_from, strrange_to = line.split(',')
                    output += 'H0={:5s} mag, n={:5s} [{:s}, {:s}]'.format(h0, n, strrange_from, strrange_to.replace('\n',''))+'\n'    
                if output == '': 
                    rest = elements[90:102].split()
                    h0, n = float(rest[0]), float(rest[1])
                    output = 'H0={:5.2f} mag, n={:5.2f} [inf, inf]'.format(h0, n)
                self.reg.setText(output)
                
                self.UpdatePlot(cobs_id, True)
                
    def UpdatePlot(self, cobs_id, new):
        # process files to array, and get a jday translations for perihleions
        self.obs_list, self.peridict, self.obscode, self.returns = npfunc.sort( koprhand.ObsFileUpdate( 'local', cobs_id, [] ), self.kometa, new, self.obscode, self.returns)
        
        if self.obs_list:
            # limits for all observations
            if new:
              limits = npfunc.getranges(self.obs_list, self.plotopt[0][3].currentIndex())
            else:
              limits = ( self.plotopt[0][1].text(), self.plotopt[0][2].text(), self.plotopt[1][1].text(), self.plotopt[1][2].text() )

            # set filter to select observations ... in case of new comet, ranges are initial limits  
            set_filter = [ self.plotopt[0][3].currentIndex(), limits[0], limits[1] ]

            # change magparams to number array +  grab additional formulas
            mp_arrray = self.GrabAditional(npfunc.magparams(self.kometa.magparams)) 

            # filtered obs list
            corr = [ self.plotopt[1][3].currentIndex(), self.settings[6], self.settings[7], self.settings[5] ]
            array = copy.deepcopy(self.obs_list)
            self.working_list = npfunc.filtered_obs( mp_arrray , array, self.peridict, self.settings, self.types, self.obscode, self.returns, set_filter, corr) 
            
            # get plot lists
            plot_points = npfunc.PlotLists( self.working_list, self.types, self.returns, self.settings[4], int(self.plotopt[0][4].isChecked()), set_filter )
            plot_lines = npfunc.PlotLines( mp_arrray, self.peridict, self.returns, self.settings[4], set_filter, self.plotopt[1][3].currentIndex()) 
            
            self.sc.ax.clear()
            for line in plot_points:
              self.sc.ax.plot( line[0], line[1], marker=line[2], color=line[3], markersize = 3, ls = 'None' )
    
            for line in plot_lines:
              self.sc.ax.plot( line[0], line[1], color=line[2], lw = 2, ls = line[3], aa = True )
            
            if new:
              numranges = npfunc.NumRanges(set_filter, self.working_list, self.plotopt[0][4].isChecked() )
            else:
              numranges = npfunc.StrNumRanges(set_filter, limits)

            str(self.adit.toPlainText()).split('\n')
            
            import matplotlib.lines as mlines

            legend = []
            for item in str(self.reg.toPlainText()).split('\n'):
              if item.startswith('H0='):
                legend.append(mlines.Line2D([], [], color='black', ls='-', lw = 2, label=item))
            for item in str(self.adit.toPlainText()).split('\n'):
              if item.startswith('H0='):
                legend.append(mlines.Line2D([], [], color='black', ls=':', lw = 2, label=item))
            
            self.sc.ax.legend(handles=legend, fontsize = 8)
              
            margin = [ datetime.timedelta(days = 10), 10, 0.1, 0.01 ]
            self.sc.ax.set_xlim( ( numranges[0] - margin[set_filter[0]], numranges[1] + margin[set_filter[0]]) )
            self.sc.ax.set_ylim( ( numranges[2] + 1, numranges[3] - 1 ) )
            self.sc.ax.grid(True)
            
            if self.plotopt[0][3].currentIndex() == 0:
              self.sc.fig.autofmt_xdate()
            
            self.sc.ax.set_title('Light curve of: '+self.kometa.name)
            self.sc.ax.set_xlabel(self.plotopt[0][3].currentText(), fontsize=9)
            self.sc.ax.set_ylabel(self.plotopt[1][3].currentText(), fontsize=9)
            self.sc.ax.tick_params(axis='both', which='major', labelsize=8)
            self.sc.fig.tight_layout()  
            self.sc.canvas.draw()
            
            if new:
              self.UpdateRanges(numranges, set_filter)
        else:
            self.sc.ax.clear()
            self.sc.canvas.draw()

    def UpdateRanges(self, numranges, set_filter): # set limits as ranges for new comet                
        if set_filter[0] == 0:
          self.plotopt[0][1].setText( numranges[0].strftime('%Y/%m/%d') )
          self.plotopt[0][2].setText( numranges[1].strftime('%Y/%m/%d') )
        else:
          self.plotopt[0][1].setText( str(numranges[0]) )
          self.plotopt[0][2].setText( str(numranges[1]) )
        self.plotopt[1][1].setText( str(numranges[2]) )
        self.plotopt[1][2].setText( str(numranges[3]) )
                
class ScrollComparison(QtGui.QScrollArea):
    """ This scroll area only handles one widget for which scrolling should be provided. """
    def __init__(self, parent=None):
        QtGui.QScrollArea.__init__(self, parent)

        self.w = Analyzer(self)
        self.setWidget(self.w)    
